<?php
/**
 * FW Gallery 8.0.0
 * @copyright (C) 2020 Fastw3b
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @link http://www.fastw3b.com/ Official website
 **/

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\MVC\Model\BaseModel;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Installer\InstallerHelper;

class fwGalleryModelAddon extends BaseModel {
	function loadAddonDescription() {
		$update_name = Factory::getApplication()->input->getString('update_name');
		if ($update_name) {
			$params = ComponentHelper::getParams('com_fwgallery');
			$buff = json_decode((string)fwgHelper::request(FWMG_UPDATE_SERVER.'/index.php?option=com_fwsales&view=fwsales&layout=get_addon_description&format=json&update_name='.urlencode($update_name).'&host='.urlencode(fwgHelper::getHostName()).'&access_code='.urlencode($params->get('update_code', ''))));
			if ($buff) {
				if (!empty($buff->msg)) {
					$this->setError($buff->msg);
				}
				if (!empty($buff->html)) {
					return $buff->html;
				}
			} else {
				$this->setError(Text::_('FWA_NO_RESPONSE_FROM_FASTWEB_WEBSITE'));
			}
		} else $this->setError(Text::_('FWA_NO_ADDON_NAME'));
	}
	function getDeals() {
		$params = ComponentHelper::getParams('com_fwgallery');
		return @json_decode($params->get('deals_data'));
	}
	function getInstalledExtensionPath($row) {
		$path = '';
		switch ($row->subtype) {
			case 'datatype' :
			$path = FWMG_TYPES_PATH;
			break;
			case 'plugin' :
			$path = FWMG_PLUGINS_PATH;
			break;
			case 'module' :
			$path = FWMG_MODULES_PATH;
			break;
			case 'design' :
			$path = FWMG_DESIGNS_PATH;
			break;
		}
		$path .= $row->element.'/'.$row->element.'.xml';
		return $path;
	}
	function getInstalledExtensionId($row) {
		$path = $this->getInstalledExtensionPath($row);
		if (file_exists($path)) {
			$db = Factory::getDBO();
			$query = '';
			switch ($row->subtype) {
				case 'datatype' :
				$query = '`type`=\'plugin\' AND `folder` = \'fwgallerytype\' AND `element`='.$db->quote($row->element);
				break;
				case 'plugin' :
				$query = '`type`=\'plugin\' AND `folder` = \'fwgallery\' AND `element`='.$db->quote($row->element);
				break;
				case 'module' :
				$query = '`type`=\'module\' AND `element`='.$db->quote($row->element);
				break;
				case 'design' :
				$query = '`type`=\'plugin\' AND `folder` = \'fwgallerytmpl\' AND `element`='.$db->quote($row->element);
				break;
			}
			if ($query) {
				$db->setQuery('SELECT extension_id FROM `#__extensions` WHERE '.$query);
				return $db->loadResult();
			}
		}
	}
	function uninstall() {
		$app = Factory::getApplication();
		$ext = (array)$app->input->getRaw('ext');

		$list = fwgHelper::getPluginsData();
		$result = array();
		foreach ($list as $row) {
			if (in_array($row->update_name, $ext)) {
				$type = '';
				switch ($row->subtype) {
					case 'plugin' :
					case 'module' :
					$type = $row->type;
					break;
					case 'datatype' :
					case 'design' :
					$type = 'plugin';
					break;
				}
				if ($type) {
					$ext_id = $this->getInstalledExtensionId($row);
					$installer = Installer::getInstance();
					if ($installer->uninstall($type, $ext_id)) {
						$this->setError(Text::sprintf('FWMG_UNINSTALLED_SUCCESFULLY', ucfirst(Text::_('FWMG_'.$row->subtype)), $row->name));
						$result[] = array(
							'update_name' => $row->update_name,
							'action' => 'remove',
						);
					}
				}
			}
		}
		return $result;
	}
	function installAll($is_update=false) {
		error_reporting(E_ALL ^ E_DEPRECATED);
		$app = Factory::getApplication();
		$addons = fwgHelper::getPluginsData();
		$qty = 0;
		if ($addons) {
			foreach ($addons as $row) {
				if ($is_update) {
					if ($row->loc_version and $row->rem_version and $row->loc_version != 'x.x.x' and $row->loc_version != $row->rem_version) {
						$app->input->set('ext', array($row->update_name));
						if ($this->install(true)) {
							$qty++;
						}
					}
				} else {
					if (!empty($row->installable)) {
						$app->input->set('ext', array($row->update_name));
						if ($this->install()) {
							$qty++;
						}
					}
				}
			}
		}
		$this->setError(Text::sprintf($is_update?'FWMG_ADDONS_UPDATED_QTY':'FWMG_ADDONS_INSTALLED_QTY', $qty));
	}
	function install($is_update=false) {
		error_reporting(E_ALL ^ E_DEPRECATED);
		$app = Factory::getApplication();
		$params = ComponentHelper::getParams('com_fwgallery');
		$code = $params->get('update_code');
		if (!$code) {
			$this->setError(Text::sprintf('FWMG_NEED_VERIFY_X_TO_UPDATE', fwgHelper::getHostName()));
			return;
		}
		$ext = $app->input->getVar('ext');
		$list = fwgHelper::getPluginsData();
		$result = array();
		foreach ($list as $row) {
			if (in_array($row->update_name, $ext)) {
				if ($row->update_name == 'mod_fwg_map') {
					$row->update_name = 'pkg_fwgallery_map';
				}
				$buff = fwgHelper::request(FWMG_UPDATE_SERVER.'/index.php?option=com_fwsales&view=updates&layout=package&format=raw&package='.urlencode($row->update_name).'&host='.urlencode(fwgHelper::getHostName()).'&access_code='.urlencode($params->get('update_code')).'&cms='.(class_exists('Joomla\CMS\Version')?'joomla':'wp').'&dummy=extension.xml');
				if (preg_match('#detailsurl="([^"]*)"#', $buff, $match)) {
					$url = str_replace('&amp;', '&', $match[1]);
					if ($buff = fwgHelper::request($url)) {
						if (preg_match('#<downloadurl[^>]*>([^<]+)</downloadurl>#', $buff, $match)) {
							$url = str_replace('&amp;', '&', $match[1]).'&is_app=1&code='.urlencode($code).'&host='.urlencode(fwgHelper::getHostName()).'&cms='.(class_exists('Joomla\CMS\Version')?'joomla':'wp');
							if ($buff = fwgHelper::request($url)) {
								$path = JPATH_SITE.'/tmp/';

								$filename = '';
								do {
									$filename = 'inst'.rand().'.zip';
								} while (file_exists($path.$filename));

								if (File::write($path.$filename, $buff)) {
									$package = InstallerHelper::unpack($path.$filename, true);
									if (!empty($package['dir'])) {
										$installer = Installer::getInstance();
										if ($installer->install($package['dir'])) {
											/* enable addon after installing/updating */
											$db = Factory::getDBO();
											$folder = '';
											$element = $row->update_name;
											if (strpos($element, 'plg_') === 0) {
												$buff = explode('_', $row->update_name, 3);
												if (count($buff) == 3) {
													$folder = $buff[1];
													$element = $buff[2];
												}
											}
											$db->setQuery('UPDATE `#__extensions` SET enabled = 1  WHERE `element` = '.$db->quote($element).($folder?(' AND `folder` = '.$db->quote($folder)):''));
											$db->execute();

											if ($row->update_name == urldecode(FWMG_UPDATE_NAME)) {
												$this->setError(Text::sprintf('FWMG_MAIN_COMPONENT_UPDATED_SUCCESFULLY', $row->name));
											} else {
												$this->setError(Text::sprintf($is_update?'FWMG_UPDATED_SUCCESFULLY':'FWMG_INSTALLED_SUCCESFULLY', ucfirst(Text::_('FWMG_'.str_replace(' ', '', $row->subtype))), $row->name, ucfirst(Text::_('FWMG_'.$row->type))));
											}
											if (file_exists($path.$filename)) File::delete($path.$filename);
											if (file_exists($package['dir'])) Folder::delete($package['dir']);
											$result[] = (object)array(
												'update_name' => $row->update_name,
												'version' => $row->rem_version,
												'action' => $is_update?'update':'install',
											);
										} else {
											$this->setError(Text::sprintf($is_update?'FWMG_NOT_UPDATED':'FWMG_NOT_INSTALLED', ucfirst(Text::_('FWMG_'.str_replace(' ', '', $row->subtype))), $row->name));
										}
										if (file_exists($package['dir'])) Folder::delete($package['dir']);
									} elseif (preg_match('#<title>([^<]+)<#msi', $buff, $match)) {
										$this->setError($row->name.' - '.$match[1]);
									}
									if (file_exists($path.$filename)) File::delete($path.$filename);
								} else $this->setError(Text::_('FWMG_CANT_WRITE_FILE'));
							} else $this->setError(Text::_('FWMG_SERVER_REFUSES_DOWNLOAD'));
						} else $this->setError(Text::_('FWMG_WRONG_RESPONSE_FORMAT_FROM_REMOTE_SERVER'));
					} else $this->setError(Text::_('FWMG_NO_RESPONSE_FROM_REMOTE_SERVER'));
				} else $this->setError(Text::_('FWMG_WRONG_RESPONSE_FORMAT_FROM_REMOTE_SERVER'));
			}
		}
		return $result;
	}
	function enable() {
		$input = Factory::getApplication()->input;
		$exts = $input->getVar('ext');
		if (!empty($exts[0])) {
			$ext = $exts[0];
			$lang = Factory::getLanguage();
			$db = Factory::getDBO();
			if (strpos($ext, 'plg_') === 0) {
				$buff = explode('_', $ext);
				if (count($buff) == 3) {
					$db->setQuery('SELECT extension_id, name FROM `#__extensions` WHERE `folder` = '.$db->quote($buff[1]).' AND `element` = '.$db->quote($buff[2]));
					$lang->load($ext, JPATH_ADMINISTRATOR);
				} else $this->setError(Text::_('FWMG_INCORRECT_ADDON_NAME'));
			} else {
				$db->setQuery('SELECT extension_id, name FROM `#__extensions` WHERE `element` = '.$db->quote($ext));
				$lang->load($ext, JPATH_SITE);
			}
			if ($extension = $db->loadObject()) {
				$db->setQuery('UPDATE `#__extensions` SET `enabled` = 1 WHERE extension_id = '.(int)$extension->extension_id);
				if ($db->execute()) {
					if ($ext == 'mod_fwg_map') {
						$db->setQuery('UPDATE `#__extensions` SET `enabled` = 1 WHERE `folder` = \'fwgallery\' AND `element` = \'map\'');
						$db->execute();
					}
					$this->setError(Text::sprintf('FWMG_ADDON_NAME_SUCCESSFULLY_ENABLED', Text::_($extension->name)));
					return array(array(
						'update_name' => $ext,
						'action' => 'enable',
					));
				} else $this->setError(Text::_('FWMG_DB_ERROR').' '.$db->getError());
			} else $this->setError(Text::_('FWMG_ADDON_NOT_FOUND'));
		} else $this->setError(Text::_('FWMG_NO_ADDON_NAME_TO_ENABLE'));
	}
	function disable() {
		$input = Factory::getApplication()->input;
		$exts = $input->getVar('ext');
		if (!empty($exts[0])) {
			$ext = $exts[0];
			$lang = Factory::getLanguage();
			$db = Factory::getDBO();
			if (strpos($ext, 'plg_') === 0) {
				$buff = explode('_', $ext);
				if (count($buff) == 3) {
					$db->setQuery('SELECT extension_id, name FROM `#__extensions` WHERE `folder` = '.$db->quote($buff[1]).' AND `element` = '.$db->quote($buff[2]));
					$lang->load($ext, JPATH_ADMINISTRATOR);
				} else $this->setError(Text::_('FWMG_INCORRECT_ADDON_NAME'));
			} else {
				$db->setQuery('SELECT extension_id, name FROM `#__extensions` WHERE `element` = '.$db->quote($ext));
				$lang->load($ext, JPATH_SITE);
			}
			if ($extension = $db->loadObject()) {
				$db->setQuery('UPDATE `#__extensions` SET `enabled` = 0 WHERE extension_id = '.(int)$extension->extension_id);
				if ($db->execute()) {
					if ($ext == 'mod_fwg_map') {
						$db->setQuery('UPDATE `#__extensions` SET `enabled` = 0 WHERE `folder` = \'fwgallery\' AND `element` = \'map\'');
						$db->execute();
					}
					$this->setError(Text::sprintf('FWMG_ADDON_NAME_SUCCESSFULLY_DISABLED', Text::_($extension->name)));
					return array(array(
						'update_name' => $ext,
						'action' => 'disable',
					));
				} else $this->setError(Text::_('FWMG_DB_ERROR').' '.$db->getError());
			} else $this->setError(Text::_('FWMG_ADDON_NOT_FOUND'));
		} else $this->setError(Text::_('FWMG_NO_ADDON_NAME_TO_DISABLE'));
	}
}
