<?php
/**
 * FW Gallery 8.0.0
 * @copyright (C) 2020 Fastw3b
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @link http://www.fastw3b.com/ Official website
 **/

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\MVC\Model\BaseModel;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;

class fwGalleryModelData extends BaseModel {
	function getPlugins($view) {
		$app = Factory::getApplication();
		PluginHelper::importPlugin('fwgallery');
		return $app->triggerEvent('onfwGetAdminForm', array('com_fwgallery', $view));
	}
	function getPlugin() {
		$input = Factory::getApplication()->input;
		if ($plugin = $input->getString('plugin'))
			return PluginHelper::getPlugin('fwgallery', $plugin);
	}
	function processPlugin(&$plugin) {
		if (PluginHelper::importPlugin('fwgallery', $plugin->name)) {
			$app = Factory::getApplication();
			$result = $app->triggerEvent('onfwProcess', array($plugin->name));
		}
	}
	function exportData() {
		$db = Factory::getDBO();
		$folder = '';
		$path = JPATH_SITE.'/tmp/';
		do {
			$folder = rand();
		} while (file_exists($path.$folder));
		Folder::create($path.$folder);

		$tables = array(
			'category',
			'category_tag',
			'file',
			'file_audio',
			'file_file',
			'file_image',
			'file_location',
			'file_tag',
			'file_video',
			'file_vote',
			'tag'
		);

		$query = '';
		foreach ($tables as $table) {
			$table_name = '#__fwsg_'.$table;
			$table = $db->getPrefix().'fwsg_'.$table;
			$db->setQuery('SHOW CREATE TABLE `'.$table.'`');
			if ($obj = $db->loadAssoc()) {
				$query .= 'DROP TABLE IF EXISTS `'.$table_name."`;\n";
				$query .= str_replace('CREATE TABLE ', 'CREATE TABLE IF NOT EXISTS ', str_replace($table, $table_name, $obj['Create Table'])).";\n\n";
			}

			$db->setQuery('SHOW COLUMNS FROM `'.$table.'`');
			if ($cols = $db->loadAssocList()) {
				foreach ($cols as $i=>$col) $cols[$i]['is_numeric'] = preg_match("/^(\w*int|year)/", $col['Type'])?true:false;
				$db->setQuery('SELECT * FROM `'.$table.'`');
				if ($list = $db->loadAssocList()) {
					foreach ($list as $row) {
						$query .= 'INSERT INTO `'.$table_name.'` VALUES (';
						foreach ($cols as $i=>$col) {
							if ($i) $query .= ',';
							if ($col['Field'] and $col['is_numeric']) $query .= (int)$row[$col['Field']];
							elseif (is_null($row[$col['Field']])) $query .= 'NULL';
							else $query .= $db->quote($row[$col['Field']]);
						}
						$query .= ");\n";
					}
				}
			}
			$query .= "\n";
		}
		File::write($path.$folder.'/gallery.sql', $query);

		$db->setQuery('SELECT id, name, username, email, password, block, registerDate, params FROM `#__users` AS u WHERE EXISTS(SELECT * FROM `#__fwsg_category` AS p WHERE p.user_id = u.id) OR EXISTS(SELECT * FROM `#__fwsg_file` AS p WHERE p.user_id = u.id)');
		if ($list = $db->loadAssocList() and $fh = fopen($path.$folder.'/users.csv', 'w')) {
			foreach ($list as $row) {
				fputcsv($fh, $row);
			}
			fclose($fh);
		}
		
		$db->setQuery('SELECT params FROM `#__extensions` WHERE `type` = \'component\' AND `element` = \'com_fwgallery\'');
		if ($params = $db->loadResult()) {
			File::write($path.$folder.'/settings.txt', $params);
		}

		return $folder;
	}
	function storeExportData() {
		ini_set('memory_limit', '2048M');
		set_time_limit(600);
		$input = Factory::getApplication()->input;
		if ($folder = $input->getString('folder')) {
			$src_path = str_replace('\\', '/', JPATH_SITE).'/tmp/'.$folder.'/';
			if (!is_dir($src_path)) {
				$this->setError(Text::_('FWMG_NO_DATA_FOLDER'));
				return;
			}
			$trg_path = str_replace('\\', '/', FWMG_STORAGE_PATH).'backups/';
			if (!file_exists($trg_path)) {
				Folder::create($trg_path);
			}
			if (!file_exists($trg_path)) {
				$this->setError(Text::_('FWMG_CANT_CREATE_EXPORT_FOLDER'));
				return;
			}

			$backup_images = $input->getInt('images');

			$files = Folder::files($src_path, '.*', true, true);
			if ($files) {
				include_once(JPATH_COMPONENT_SITE.'/helpers/tar.php');
				
				$filename = 'fwmg_data_'.($backup_images?'files_':'').date('d_M_Y_His').'.tar';
				$tar = new tar($trg_path.$filename, $src_path);
				foreach ($files as $file) {
					$tar->add($file);
				}

				if ($backup_images) {
					$img_src_path = str_replace('\\', '/', FWMG_STORAGE_PATH);
					$files = Folder::files($img_src_path, '.*', true, true);
					if ($files) {
						foreach ($files as $file) {
							if (preg_match('#fwmg_data.*(zip|tar)$#', $loc_path)) {
								continue;
							}
							$loc_path = str_replace($img_src_path, '', str_replace('\\', '/', $file));
							$tar->add($file, $loc_path);
						}
					}
				}

				if ($tar->write()) {
					Folder::delete($src_path);
					return array(
						'name' => $filename,
						'size' => fwgHelper::humanFileSize(filesize($trg_path.$filename))
					);
				} else {
					Folder::delete($src_path);
				}
			} else {
				$this->setError(Text::_('FWMG_NO_DATA_EXPORT'));
			}
		} else {
			$this->setError(Text::_('FWMG_NO_FOLDER_TO_EXPORT'));
		}
	}
	function getBackupsList() {
		$src_path = str_replace('\\', '/', FWMG_STORAGE_PATH).'backups';
		return is_dir($src_path)?Folder::files($src_path, '\.(zip|tar)$'):array();
	}
	function backupRestore() {
		ini_set('memory_limit', '2048M');
		set_time_limit(600);
		$input = Factory::getApplication()->input;
		if ($filename = str_replace(array('/', '\\'), '', $input->getString('filename'))) {
			$path = FWMG_STORAGE_PATH.'backups/';
			if (file_exists($path.$filename)) {
				$trg_path = FWMG_STORAGE_PATH;
				if (strpos('_data_images_', $filename) !== false) {
					$dirs = Folder::folders($trg_path);
					foreach ($dirs as $dir) {
						if (is_dir($trg_path.$dir)) {
							Folder::delete($trg_path.$dir);
						}
					}
				}

				if (!file_exists($trg_path)) {
					mkdir($trg_path);
				}
				$archive_possible_unpacked = false;
				if (strtolower(File::getExt($filename) == 'zip')) {
					$zip = new ZipArchive;
					if ($zip->open($path.$filename) === TRUE) {
						$zip->extractTo($trg_path);
						$zip->close();
						$archive_possible_unpacked = true;
					}
				} else {
					include_once(JPATH_COMPONENT_SITE.'/helpers/untar.php');
					$tarfile = new untar($path.$filename);
					if ($filelist = $tarfile->getfilelist()) {
						for ($x = 0; $x < count($filelist); $x++) {
							$filename = $trg_path.$filelist[$x]["filename"];
							if ($filelist[$x]["filetype"] == "directory") {
								mkdir($filename);
							} else {
								$dirname = dirname($filename);
								if (!file_exists($dirname)) {
									mkdir($dirname, 0777, true);
								}
						
								$file = fopen($filename, "w");
								$data = $tarfile->extract($filelist[$x]["filename"]);
								fwrite($file, $data);
								fclose($file);
							}
						}
						$archive_possible_unpacked = true;
					}
				}
				if ($archive_possible_unpacked) {
					$db = Factory::getDBO();
					if (file_exists($trg_path.'gallery.sql')) {
						$buff = file_get_contents($trg_path.'gallery.sql');
						$queries = fwgHelper::splitSql($buff);

						$err = $total = 0;
						foreach ($queries as $query) {
							if (trim($query)) {
								$db->setQuery($query);
								if (!$db->execute()) {
									$err++;
								}
								$total++;
							}
						}

						File::delete($trg_path.'gallery.sql');
					}
					if (file_exists($trg_path.'settings.txt')) {
						$params = ComponentHelper::getParams('com_fwgallery');
						$params->loadFile($trg_path.'settings.txt');
						fwgHelper::storeConfig($params);
						File::delete($trg_path.'settings.txt');
					}
					if (file_exists($trg_path.'users.csv')) {
						if ($fh = fopen($trg_path.'users.csv', 'r')) {
							while ($row = fgetcsv($fh, 10000)) {
								/* id, name, username, email, password, block, registerDate, params */
								$data = array(
									'name' => $row[1],
									'username' => $row[2],
									'email' => $row[3],
									'password' => $row[4],
									'block' => $row[5],
									'registerDate' => $row[6],
									'params' => json_decode($row[7], true),
									'groups' => array(2)
								);
								$db->setQuery('SELECT id FROM `#__users` AS u WHERE u.email = '.$db->quote($data['email']).' OR u.username = '.$db->quote($data['email']));
								$uid = $db->loadResult();
								if (!$uid) {
									$user = clone(Factory::getUser(0));
									if ($user->bind($data) and $user->save()) {
										$uid = $user->id;
									}
								}
								if ($uid and $uid != $row[0]) {
									$tables = array('category', 'file');
									foreach ($tables as $table) {
										$db->setQuery('UPDATE `#__fwsg_'.$table.'` SET user_id = '.(int)$uid.' WHERE user_id = '.(int)$row[0]);
										$db->execute();
									}
								}
							}
							fclose($fh);
						}
						File::delete($trg_path.'users.csv');
					}
					return true;
				}
			} else {
				$this->setError(Text::_('FWMG_FILE_NOT_FOUND_TO_RESTORE'));
			}
		} else {
			$this->setError(Text::_('FWMG_NO_FILENAME_TO_RESTORE'));
		}
	}
	function backupDelete() {
		$input = Factory::getApplication()->input;
		if ($filename = str_replace(array('/', '\\'), '', $input->getString('filename'))) {
			$path = FWMG_STORAGE_PATH.'backups/';
			if (file_exists($path.$filename)) {
				if (File::delete($path.$filename)) {
					return true;
				} else {
					$this->setError(Text::_('FWMG_CANT_DELETE_FILE'));
				}
			} else {
				$this->setError(Text::_('FWMG_FILE_NOT_FOUND_TO_DELETE'));
			}
		} else {
			$this->setError(Text::_('FWMG_NO_FILENAME_TO_DELETE'));
		}
	}
	function backupUpload() {
		$input = Factory::getApplication()->input;
		if ($file = $input->files->get('upload', array(), 'raw') and !empty($file['name']) and empty($file['error'])) {
			$path = FWMG_STORAGE_PATH.'backups/';
			if (!file_exists($path)) {
				Folder::create($path);
			}
			if (File::copy($file['tmp_name'], $path.$file['name'])) {
				return array(
					'name' => $file['name'],
					'size' => fwgHelper::humanFileSize($file['size'])
				);
			} else {
				$this->setError(Text::_('FWMG_CANT_COPY_UPLOADED_FILE'));
			}
		} else {
			$this->setError(Text::_('FWMG_NO_FILE_TO_UPLOAD'));
		}
	}
}
