<?php
/**
 * FW Gallery 8.0.0
 * @copyright C 2019 Fastw3b
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @link http://www.fastw3b.com/ Official website
 **/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Model\BaseModel;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Pagination\Pagination;
use Joomla\Utilities\ArrayHelper;

class fwGalleryModelTranslation extends BaseModel {
    function getUserState($name, $def='', $type='cmd') {
        $app = Factory::getApplication();
        $context = 'com_fwgallery.language.';
        return $app->getUserStateFromRequest($context.$name, $name, $def, $type);
    }
	function checkTranslations() {
		fwgHelper::getPluginsData();

		$params = ComponentHelper::getParams('com_fwgallery');
		$params_changed = false;
		if ($remote_langs = json_decode($params->get('langs_data', '[]'))) {
			$list = fwgHelper::getInstalledLanguages();
			foreach ($list as $row) {
				if ($params->get('translation_requested_'.$row->tag) and !empty($remote_langs->{$row->tag})) {
					$params->set('translation_requested_'.$row->tag, 0);
					$params_changed = true;
				}
			}
		}
		if ($params_changed) {
			fwgHelper::storeConfig($params);
		}
	}
	function getLanguageData($ext) {
		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());

		$search = $this->getUserState('search', '', 'string');

		$path = '';
		$filename = '';

		switch ($ext->type) {
			case 'component' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = $ext->_element;
			break;
			case 'module' :
			case 'template' :
				$path = FWMG_LANGUAGE_SITE;
				$filename = $ext->_element;
			break;
			case 'plugin' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = 'plg_'.$ext->folder.'_'.$ext->_element;
			break;
		}

		$src_file = $path.'/language/en-GB/en-GB.'.$filename.'.ini';
		$src_buff = file_exists($src_file)?explode("\n", file_get_contents($src_file)):array();
		$src_data = array();
		$result = array();
		foreach ($src_buff as $i=>$row) if (trim($row)) {
			$row = explode("=", trim($row), 2);
			if (count($row) < 2) continue;
			$row[1] = preg_replace('#^"|"$#', '', $row[1]);
			$src_data[$row[0]] = $row[1];
			if (!empty($row[1]) and (!$search or ($search and (mb_stripos($row[0], $search) !== false or mb_stripos($row[1], $search) !== false)))) {
				$result[$row[0]] = array(
					'src' => $row[1],
					'trg' => ''
				);
			}
		}

		$trg_file = $path.'/language/'.$lang->tag.'/'.$lang->tag.'.'.$filename.'.ini';
		$trg_buff = file_exists($trg_file)?explode("\n", file_get_contents($trg_file)):array();
		foreach ($trg_buff as $i=>$row) if (trim($row)) {
			$row = explode("=", trim($row), 2);
			if (!empty($row[1]) and (!$search or ($search and (mb_stripos($row[0], $search) !== false or mb_stripos($row[1], $search) !== false)))) {
				if (isset($result[$row[0]])) {
					$result[$row[0]]['trg'] = preg_replace('#^"|"$#', '', $row[1]);
				} else {
					$found = false;
					foreach ($src_buff as $srow) {
						$srow = explode('=', trim($srow), 2);
						if ($srow[0] == $row[0]) {
							$found = true;
							$result[$row[0]] = array(
								'src' => trim($srow[1], '"'),
								'trg' => preg_replace('#^"|"$#', '', $row[1])
							);
							break;
						}
					}
					if (!$found) {
						$result[$row[0]] = array(
							'src' => '',
							'trg' => preg_replace('#^"|"$#', '', $row[1])
						);
					}
				}
			}
		}

		/* load overrides */
		$trg_file = $path.'/language/overrides/'.$lang->tag.'.override.ini';
		$trg_buff = file_exists($trg_file)?explode("\n", file_get_contents($trg_file)):array();
		foreach ($trg_buff as $i=>$row) if (trim($row)) {
			$row = explode("=", trim($row), 2);
			$found = ($search and (mb_stripos($row[0], $search) !== false or mb_stripos($row[1], $search) !== false));
			if (!empty($row[1]) and (isset($result[$row[0]]) or !$search or $found)) {
				if (isset($result[$row[0]]) or ($found and !empty($src_data[$row[0]]))) {
					if (!isset($result[$row[0]])) {
						$result[$row[0]] = array(
							'src' => $src_data[$row[0]],
							'trg' => ''
						);
					}
					$result[$row[0]]['trg'] = preg_replace('#^"|"$#', '', $row[1]);
					$result[$row[0]]['override'] = true;
				}
			}
		}

		return $result;
	}
	function getLanguageOverridesFilename() {
		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());
		$exts = $this->loadExtensions();
		$ext = $this->getExtension($exts, $this->getUserState('extension', 'com_fwgallery', 'string'));
		$path = '';
		switch ($ext->type) {
			case 'component' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
			break;
			case 'module' :
			case 'template' :
				$path = FWMG_LANGUAGE_SITE;
			break;
			case 'plugin' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
			break;
		}

		return $path.'/language/overrides/'.$lang->tag.'.override.ini';
	}
	function getLanguageFilename() {
		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());
		$exts = $this->loadExtensions();
		$ext = $this->getExtension($exts, $this->getUserState('extension', 'com_fwgallery', 'string'));

		$path = '';
		$filename = '';

		switch ($ext->type) {
			case 'component' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = $ext->_element;
			break;
			case 'module' :
			case 'template' :
				$path = FWMG_LANGUAGE_SITE;
				$filename = $ext->_element;
			break;
			case 'plugin' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = 'plg_'.$ext->folder.'_'.$ext->_element;
			break;
		}

		return $path.'/language/'.$lang->tag.'/'.$lang->tag.'.'.$filename.'.ini';
	}
	function copy() {
		$input = Factory::getApplication()->input;

		$cid = $input->getVar('cid');
		$lang_data = $input->getVar('lang_data');

		$tag = $input->getCmd('tag');
		$text = $input->gethtml('text');

		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());

		$exts = $this->loadExtensions();
		$ext = $this->getExtension($exts, $this->getUserState('extension', 'com_fwgallery', 'string'));

		$path = '';
		switch ($ext->type) {
			case 'component' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
			break;
			case 'module' :
			case 'template' :
				$path = FWMG_LANGUAGE_SITE;
			break;
			case 'plugin' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
			break;
		}

		$trg_file = $path.'/language/overrides/'.$lang->tag.'.override.ini';

		$result = array();
/* load existing values */
		$trg_buff = file_exists($trg_file)?explode("\n", file_get_contents($trg_file)):array();
		foreach ($trg_buff as $i=>$row) {
			if (trim($row)) {
				$row = explode("=", trim($row), 2);
				if (!empty($row[1])) {
					$result[$row[0]] = preg_replace('#^"|"$#', '', $row[1]);
				}
			}
		}

		if (is_array($cid) and is_array($lang_data)) {
			foreach ($cid as $tag) {
				if (isset($lang_data[$tag])) {
					$result[$tag] = $lang_data[$tag];
				}
			}
		}

		$buff = '';
		foreach ($result as $const=>$val) {
			$buff .= $const.'="'.str_replace(array("\r", "\n", "\t"), ' ', $val).'"'."\n";
		}

/* store updated language file */
		if (File::write($trg_file, $buff)) {
			$this->setError(Text::_('FWMG_LANGUAGE_FILE_SUCCESFULLY_UPDATED'));
			return true;
		} else {
			$this->setError(Text::_('FWMG_LANGUAGE_CANT_WRITE_FILE'));
		}

	}
	function save() {
		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());
		$input = Factory::getApplication()->input;

		$exts = $this->loadExtensions();
		$ext = $this->getExtension($exts, $this->getUserState('extension', 'com_fwgallery', 'string'));

		$path = '';
		$filename = '';

		switch ($ext->type) {
			case 'component' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = $ext->_element;
			break;
			case 'module' :
			case 'template' :
				$path = FWMG_LANGUAGE_SITE;
				$filename = $ext->_element;
			break;
			case 'plugin' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = 'plg_'.$ext->folder.'_'.$ext->_element;
			break;
		}

		$trg_file = $path.'/language/'.$lang->tag.'/'.$lang->tag.'.'.$filename.'.ini';

		$result = array();
/* load existing values */
		$trg_buff = file_exists($trg_file)?explode("\n", file_get_contents($trg_file)):array();
		foreach ($trg_buff as $i=>$row) {
			if (trim($row)) {
				$row = explode("=", trim($row), 2);
				if (!empty($row[1])) {
					$result[$row[0]] = preg_replace('#^"|"$#', '', $row[1]);
				}
			}
		}

/* update changes */
		if ($data = (array)$input->getRaw('lang_data')) {
			foreach ($data as $const => $val) {
				$result[$const] = $val;
			}

/* collect language file */
			$buff = "\n";
			foreach ($result as $const=>$val) {
				$buff .= $const.'="'.str_replace(array("\r", "\n", "\t"), ' ', $val).'"'."\n";
			}

/* store updated language file */
			if (File::write($trg_file, $buff)) {
				$this->setError(Text::_('FWMG_LANGUAGE_FILE_SUCCESFULLY_UPDATED'));
				return true;
			} else {
				$this->setError(Text::_('FWMG_LANGUAGE_CANT_WRITE_FILE'));
			}
		}
	}
	function getExtension($exts, $ext_element) {
		foreach ($exts as $ext) {
			if ($ext->element == $ext_element) {
				return $ext;
			}
		}
		return $exts[0];
	}
	function loadExtensions() {
		$db = Factory::getDBO();
		$db->setQuery('SELECT `element`, `folder`, type, name, client_id FROM `#__extensions` WHERE (`element` LIKE \'%fwg%\' OR `folder` = \'fwgallery\' OR `folder` = \'fwgallerytype\' OR `folder` = \'fwgallerytmpl\') ORDER BY `element`');
		if ($list = $db->loadObjectList()) {
			foreach ($list as $i => $row) {
				$list[$i]->_name = $row->name;
				$list[$i]->_element = $row->element;
/*				if ($row->folder) {
					$list[$i]->element .= '-'.$row->folder;
				}*/

				$path = '';
				if (($row->type == 'template' or $row->type == 'module') and $row->client_id == 0) {
					$path = FWMG_LANGUAGE_SITE.'/language/en-GB/en-GB.';
				} else {
					$path = FWMG_LANGUAGE_ADMINISTRATOR.'/language/en-GB/en-GB.';
				}
				$name = '';
				if ($row->type == 'plugin') {
					$filename = $path.'plg_'.strtolower($row->folder).'_'.strtolower($row->element).'.sys.ini';
				} else {
					$filename = $path.strtolower($row->element).'.sys.ini';
				}

				if (file_exists($filename)) {
					$buff = file_get_contents($filename);
					if (preg_match('#'.$row->name.'\="(.+)"#i', $buff, $match)) {
						$name = $match[1];
					} elseif (preg_match('#FW_REALESTATE_'.$row->element.'\="(.+)"#i', $buff, $match)) {
						$name = $match[1];
					}
				}
				if ($name) {
					$list[$i]->name = $name;
				}
				if ($row->type) {
					$list[$i]->name .= ' ['.Text::_('FWMG_'.$row->type).']';
				}
			}

			ArrayHelper::sortObjects($list, array('name'), $direction = 1, $caseSensitive = false);

			return $list;
		}
	}
	function getPagination(&$data) {
		$app = Factory::getApplication();

		$limitstart = (int)$app->input->getInt('limitstart');
		$limit = (int)$app->input->getInt('limit', $app->getCfg('list_limit', 20));
		$qty = count($data);

		if ($qty > $limit) {
			$data = array_slice($data, $limitstart, $limit);
		}

		return new Pagination(
			$qty,
			$limitstart,
			$limit
		);
	}
	function request($is_reinstall=false) {
		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());
		$params = ComponentHelper::getParams('com_fwgallery');
		$exts = $this->loadExtensions();
		$ext = $this->getExtension($exts, $this->getUserState('extension', 'com_fwgallery', 'string'));
		$version = '';

		$path = array();
		$filename = '';
		$xml_filename = '';
		$result = false;

		switch ($ext->type) {
			case 'component' :
				$path['site'] = FWMG_LANGUAGE_SITE;
				$path['admin'] = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = $ext->_element;
				$xml_filename = '/administrator/components/'.$filename.'/'.str_replace('com_', '', $filename).'.xml';
			break;
			case 'module' :
				$path['site'] = FWMG_LANGUAGE_SITE;
				$filename = $ext->_element;
				$xml_filename = '/modules/'.$filename.'/'.$filename.'.xml';
			break;
			case 'template' :
				$path['site'] = FWMG_LANGUAGE_SITE;
				$filename = $ext->_element;
				$xml_filename = '/templates/'.$filename.'/templateDetails.xml';
			break;
			case 'plugin' :
				$path['admin'] = FWMG_LANGUAGE_ADMINISTRATOR;
				$filename = 'plg_'.$ext->folder.'_'.$ext->_element;
				$xml_filename = '/plugins/'.$ext->folder.'/'.$filename.'/'.$filename.'.xml';
			break;
		}
		if ($xml_filename) {
			$xml_filename = JPATH_SITE.$xml_filename;
			if (file_exists($xml_filename)) {
				$buff = file_get_contents($xml_filename);
				if (preg_match('#<version>(.*)</version>#i', $buff, $match)) {
					$version = $match[1];
				}
			}
		}
		$data = json_decode(fwgHelper::request(FWMG_UPDATE_SERVER.'/index.php?option=com_fwtranslation&view=fwtranslation&layout=translate&format=json&access_code='.$params->get('update_code').'&ext='.$filename.'&version='.$version.'&lang='.$lang->tag.'&cms='.(class_exists('Joomla\CMS\Version')?'joomla':'wp')));

		if (!empty($data->result)) {
			$qty = 0;
			foreach ($data->result as $type=>$files) {
				foreach ($files as $lang_filename=>$text) {
					if (preg_match('#\.ini$#', $lang_filename) and !empty($path[$type])) {
						$trg_file = $path[$type].'/language/'.$lang->tag.'/'.$lang_filename;
						$folder = dirname($trg_file);
						if (!file_exists($folder)) {
							Folder::create($folder);
						}
						if (file_exists($folder) and file_put_contents($trg_file, $text)) {
							$qty++;
						}
					}
				}
			}
			if ($qty) {
				$result = true;
				$this->setError(Text::_($is_reinstall?'FWMG_LANGUAGE_FILES_REINSTALLED':'FWMG_LANGUAGE_FILES_INSTALLED'));
			} else {
				$this->setError(Text::_('FWMG_NO_FILES_RECEIVED_FROM_TRANSLATION_SERVER'));
			}
		} else {
			$lang = false;
			if (!empty($data->msg)) {
				$this->setError($data->msg);
			} else {
				$this->setError(Text::_('FWMG_NO_RESPONSE_FROM_TRANSLATION_SERVER'));
			}
		}
		if ($lang) {
			$params->set('translation_requested_'.$lang->tag, 1);
			fwgHelper::storeConfig($params);
		}
		return $result;
	}
	function upload() {
		$langs = fwgHelper::getInstalledLanguages();
		$lang = ArrayHelper::getValue($langs, fwgHelper::getLanguage());
		$params = ComponentHelper::getParams('com_fwgallery');
		$exts = $this->loadExtensions();
		$ext = $this->getExtension($exts, $this->getUserState('extension', 'com_fwgallery', 'string'));
		// $prefix = class_exists('Joomla\CMS\Version')?'':'.wp';
		$version = '';

		$updatename = '';
		$xml_filename = '';

		switch ($ext->type) {
			case 'component' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$updatename = $ext->_element;
				$xml_filename = '/administrator/components/'.$updatename.'/'.str_replace('com_', '', $updatename).'.xml';
			break;
			case 'module' :
				$path = FWMG_LANGUAGE_SITE;
				$updatename = $ext->_element;
				$xml_filename = '/modules/'.$updatename.'/'.$updatename.'.xml';
			break;
			case 'template' :
				$path = FWMG_LANGUAGE_SITE;
				$updatename = $ext->_element;
				$xml_filename = '/templates/'.$updatename.'/templateDetails.xml';
			break;
			case 'plugin' :
				$path = FWMG_LANGUAGE_ADMINISTRATOR;
				$updatename = 'plg_'.$ext->folder.'_'.$ext->_element;
				$xml_filename = '/plugins/'.$ext->folder.'/'.$updatename.'/'.$updatename.'.xml';
			break;
		}
		if ($xml_filename) {
			$xml_filename = JPATH_SITE.$xml_filename;
			if (file_exists($xml_filename)) {
				$buff = file_get_contents($xml_filename);
				if (preg_match('#<version>(.*)</version>#i', $buff, $match)) {
					$version = $match[1];
				}
			}
		}

		$filename = $path.'/language/overrides/'.$lang->tag.'.override.ini';
		if ($filename and file_exists($filename)) {
			$params = ComponentHelper::getParams('com_fwgallery');
			$data = array(
				'format' => 'json',
				'ext' => $updatename,
				'is_site' => 0,
				'lang' => $lang->tag,
				'version' => $version,
				'access_code' => $params->get('update_code'),
				'cms' => (class_exists('Joomla\CMS\Version')?'joomla':'wp'),
				'file' => function_exists('curl_file_create')?curl_file_create($filename):('@'.realpath($filename))
			);
			$buff = json_decode(fwgHelper::request(
				FWMG_UPDATE_SERVER.'/index.php?option=com_fwtranslation&view=fwtranslation&layout=upload',
				'post',
				$data,
				array('Content-Type'=>'multipart/form-data')
			));
			if (!empty($buff->msg)) {
				$this->setError($buff->msg);
			}
			if (!empty($buff->result)) {
				$this->setError(Text::_('FWMG_LANGUAGE_FILE_SUCCESFULLY_UPLOADED'));
				return $buff->result;
			}
		} else $this->setError(Text::_('FWMG_LANGUAGE_FILE_NOT_FOUND'));
	}
}
