<?php
/**
 * @package PHPClassCollection
 * @subpackage Tar
 * @link classes
 * @author Dennis Wronka <reptiler@users.sourceforge.net>
 */
/**
 * @package PHPClassCollection
 * @subpackage Tar
 * @link classes
 * @author Dennis Wronka <reptiler@users.sourceforge.net>
 * @version 1.1
 * @license http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html LGPL 2.1
 */
class tar
{
/**
 * The name of the tar-file to create.
 *
 * @var string
 */
var $filename;
var $filesroot;
/**
 * The list of files to add to the archive.
 *
 * @var array
 */
var $filelist=array();

/**
 * Constructor
 *
 * @param string $filename
 */
function __construct($filename, $filesroot)
{
    $this->filename=$filename;
	$this->filesroot = $filesroot;
}

/**
 * Add a file.
 *
 * @param string $filename
 */
function add($filename, $inner_filename=null)
{
    if ((file_exists($filename)) && (is_readable($filename)))
    {
        if (is_null($inner_filename)) {
            $this->filelist[]=$filename;
        } else {
            $this->filelist[]=array('name'=>$filename, 'inner_name'=>$inner_filename);
        }
    }
}

/**
 * Write the tar-file.
 *
 * @return bool
 */
function write()
{
    // sort($this->filelist);
    $tarfile=@fopen($this->filename,'w');
    if ($tarfile==false)
    {
        return false;
    }
    for ($x=0; $x<count($this->filelist); $x++)
    {
        $file = $this->filelist[$x];
        $filename = '';
        if (is_array($file)) {
            $filename = $file['inner_name'];
            $file = $file['name'];
        } else {
            $filename = str_replace($this->filesroot, '', $file);
        }
        if ((is_dir($file)) && (substr($file,-1)!='/'))
        {
            $filename.='/';
        }
        while (strlen($filename)<100)
        {
            $filename.=chr(0);
        }
        $permissions=sprintf('%o',fileperms($file)).chr(0);
        while (strlen($permissions)<8)
        {
            $permissions='0'.$permissions;
        }
        $userid=sprintf('%o',fileowner($file)).chr(0);
        while (strlen($userid)<8)
        {
            $userid='0'.$userid;
        }
        $groupid=sprintf('%o',filegroup($file)).chr(0);
        while (strlen($groupid)<8)
        {
            $groupid='0'.$groupid;
        }
        if (is_dir($file))
        {
            $filesize='0'.chr(0);
        }
        else
        {
            $filesize=sprintf('%o',filesize($file)).chr(0);
        }
        while (strlen($filesize)<12)
        {
            $filesize='0'.$filesize;
        }
        $modtime=sprintf('%o',filectime($file)).chr(0);
        $checksum='        ';
        if (is_dir($file))
        {
            $indicator=5;
        }
        else
        {
            $indicator=0;
        }
        $linkname='';
        while (strlen($linkname)<100)
        {
            $linkname.=chr(0);
        }
        $ustar='ustar  '.chr(0);
        if (function_exists('posix_getpwuid'))
        {
            $user=@posix_getpwuid(octdec($userid));
            $user=@$user['name'];
        }
        else
        {
            $user='';
        }
        while (strlen($user)<32)
        {
            $user.=chr(0);
        }
        if (function_exists('posix_getgrgid'))
        {
            $group=@posix_getgrgid(octdec($groupid));
            $group=@$group['name'];
        }
        else
        {
            $group='';
        }
        while (strlen($group)<32)
        {
            $group.=chr(0);
        }
        $devmajor='';
        while (strlen($devmajor)<8)
        {
            $devmajor.=chr(0);
        }
        $devminor='';
        while (strlen($devminor)<8)
        {
            $devminor.=chr(0);
        }
        $prefix='';
        while (strlen($prefix)<155)
        {
            $prefix.=chr(0);
        }
        $header=$filename.$permissions.$userid.$groupid.$filesize.$modtime.$checksum.$indicator.$linkname.$ustar.$user.$group.$devmajor.$devminor.$prefix;
        while (strlen($header)<512)
        {
            $header.=chr(0);
        }
        $checksum=0;
        for ($y=0;$y<strlen($header);$y++)
        {
            $checksum+=ord($header[$y]);
        }
        $checksum=sprintf('%o',$checksum).chr(0).' ';
        while (strlen($checksum)<8)
        {
            $checksum='0'.$checksum;
        }
        $header=$filename.$permissions.$userid.$groupid.$filesize.$modtime.$checksum.$indicator.$linkname.$ustar.$user.$group.$devmajor.$devminor.$prefix;
        while (strlen($header)<512)
        {
            $header.=chr(0);
        }
        fwrite($tarfile,$header);
        if ($indicator==0)
        {
            $contentfile=fopen($file,'r');
            $data=fread($contentfile,filesize($file));
            while (strlen($data)%512!=0)
            {
                $data.=chr(0);
            }
            fwrite($tarfile,$data);
        }
    }
    fclose($tarfile);
    return true;
}
}
